/* Copyright (C) 2005, Thorvald Natvig <thorvald@natvig.com>

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   - Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
   - Neither the name of the Mumble Developers nor the names of its
     contributors may be used to endorse or promote products derived from this
     software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _PLAYER_H
#define _PLAYER_H

#include <QString>
#include <QObject>
#include <QHash>
#include <QReadWriteLock>

class Channel;

class Player : public QObject {
	Q_OBJECT
  	public:
  		enum State { Connected, Authenticated };
  		State sState;
		short sId;
		int iId;
		QString qsName;
		bool bMute, bDeaf, bSuppressed;
		bool bSelfMute, bSelfDeaf;
		bool bTalking;
		Channel *cChannel;

		static QHash<short, Player *> c_qmPlayers;
		static QReadWriteLock c_qrwlPlayers;

		Player(QObject *p = NULL);
		static Player *get(short);
		static Player *add(short, QObject *p = NULL);
		static Player *match(const Player *p, bool matchname = false);
		static void remove(short);
		static void remove(Player *);
	public slots:
		void setTalking(bool talking);
		void setMute(bool mute);
		void setDeaf(bool deaf);
		void setSelfMuteDeaf(bool mute, bool deaf);
	signals:
		void talkingChanged(bool talking);
		void muteDeafChanged();
};

#else
class Player;
#endif
